@@ -137,6 +137,11 @@ struct _CreateFromLocationAsyncData {
 
     gsize offset;
     gsize length;
+
+    gchar *volume;
+    gchar *system;
+    gchar *application;
+    gchar *publisher;
 };
 
 static void create_from_location_async_data_free
@@ -144,6 +149,10 @@ static void create_from_location_async_data_free
 {
    g_object_unref(data->file);
    g_object_unref(data->res);
+   g_free(data->volume);
+   g_free(data->system);
+   g_free(data->application);
+   g_free(data->publisher);
 
    g_slice_free(CreateFromLocationAsyncData, data);
 }
@@ -193,6 +202,7 @@ G_DEFINE_TYPE(OsinfoMedia, osinfo_media, OSINFO_TYPE_ENTITY);
 struct _OsinfoMediaPrivate
 {
     GWeakRef os;
+    OsinfoInstallScriptList *scripts;
 };
 
 enum {
@@ -421,6 +431,10 @@ osinfo_media_set_property(GObject      *object,
 static void
 osinfo_media_finalize(GObject *object)
 {
+    OsinfoMedia *media = OSINFO_MEDIA(object);
+
+    g_object_unref(media->priv->scripts);
+
     /* Chain up to the parent class */
     G_OBJECT_CLASS(osinfo_media_parent_class)->finalize(object);
 }
@@ -694,6 +708,7 @@ osinfo_media_init(OsinfoMedia *media)
 {
     media->priv = OSINFO_MEDIA_GET_PRIVATE(media);
     g_weak_ref_init(&media->priv->os, NULL);
+    media->priv->scripts = osinfo_install_scriptlist_new();
 }
 
 OsinfoMedia *osinfo_media_new(const gchar *id,
@@ -803,18 +818,18 @@ create_from_location_async_data(CreateFromLocationAsyncData *data)
                             OSINFO_MEDIA_PROP_URL,
                             uri);
     g_free(uri);
-    if (!is_str_empty(data->pvd.volume))
+    if (!is_str_empty(data->volume))
         osinfo_entity_set_param(OSINFO_ENTITY(media),
                                 OSINFO_MEDIA_PROP_VOLUME_ID,
-                                data->pvd.volume);
-    if (!is_str_empty(data->pvd.system))
+                                data->volume);
+    if (!is_str_empty(data->system))
         osinfo_entity_set_param(OSINFO_ENTITY(media),
                                 OSINFO_MEDIA_PROP_SYSTEM_ID,
-                                data->pvd.system);
-    if (!is_str_empty(data->pvd.publisher))
+                                data->system);
+    if (!is_str_empty(data->publisher))
         osinfo_entity_set_param(OSINFO_ENTITY(media),
                                 OSINFO_MEDIA_PROP_PUBLISHER_ID,
-                                data->pvd.publisher);
+                                data->publisher);
     if (!is_str_empty(data->pvd.application))
         osinfo_entity_set_param(OSINFO_ENTITY(media),
                                 OSINFO_MEDIA_PROP_APPLICATION_ID,
@@ -1153,19 +1168,19 @@ static void on_pvd_read(GObject *source,
         return;
     }
 
-    data->pvd.volume[MAX_VOLUME - 1] = 0;
-    g_strchomp(data->pvd.volume);
+    data->volume = g_strndup(data->pvd.volume, MAX_VOLUME);
+    g_strchomp(data->volume);
 
-    data->pvd.system[MAX_SYSTEM - 1] = 0;
-    g_strchomp(data->pvd.system);
+    data->system = g_strndup(data->pvd.system, MAX_SYSTEM);
+    g_strchomp(data->system);
 
-    data->pvd.publisher[MAX_PUBLISHER - 1] = 0;
-    g_strchomp(data->pvd.publisher);
+    data->publisher = g_strndup(data->pvd.publisher, MAX_PUBLISHER);
+    g_strchomp(data->publisher);
 
-    data->pvd.application[MAX_APPLICATION - 1] = 0;
-    g_strchomp(data->pvd.application);
+    data->application = g_strndup(data->pvd.application, MAX_APPLICATION);
+    g_strchomp(data->application);
 
-    if (is_str_empty(data->pvd.volume)) {
+    if (is_str_empty(data->volume)) {
         g_set_error(&error,
                     OSINFO_MEDIA_ERROR,
                     OSINFO_MEDIA_ERROR_INSUFFICIENT_METADATA,
@@ -1655,7 +1670,8 @@ gboolean osinfo_media_supports_installer_script(OsinfoMedia *media)
     os = osinfo_media_get_os(media);
     list = osinfo_os_get_install_script_list(os);
 
-    if (osinfo_list_get_length(OSINFO_LIST(list)) == 0) {
+    if (osinfo_list_get_length(OSINFO_LIST(list)) == 0 &&
+        osinfo_list_get_length(OSINFO_LIST(media->priv->scripts)) == 0) {
         ret = FALSE;
         goto cleanup;
     }
@@ -1670,6 +1686,36 @@ gboolean osinfo_media_supports_installer_script(OsinfoMedia *media)
     return ret;
 }
 
+/**
+ * osinfo_media_add_install_script:
+ * @media: an #OsinfoMedia instance
+ * @script: an #OsinfoInstallScript instance
+ *
+ * Adds an @script to the specified @media
+ */
+void osinfo_media_add_install_script(OsinfoMedia *media, OsinfoInstallScript *script)
+{
+    g_return_if_fail(OSINFO_IS_MEDIA(media));
+
+    osinfo_list_add(OSINFO_LIST(media->priv->scripts), OSINFO_ENTITY(script));
+}
+
+/**
+ * osinfo_media_get_install_script_list:
+ * @media: an #OsinfoMedia instance
+ *
+ * Returns: (transfer full): a list of the install scripts for the specified media
+ */
+OsinfoInstallScriptList *osinfo_media_get_install_script_list(OsinfoMedia *media)
+{
+    OsinfoList *new_list;
+
+    g_return_val_if_fail(OSINFO_IS_MEDIA(media), NULL);
+    new_list = osinfo_list_new_copy(OSINFO_LIST(media->priv->scripts));
+
+    return OSINFO_INSTALL_SCRIPTLIST(new_list);
+}
+
 /*
  * Local variables:
  *  indent-tabs-mode: nil
